@@ -6,8 +6,8 @@ use Mock::Inflate::Name;
 
 table {
     name 'mock_inflate';
-    pk 'id';
-    columns qw/ id name foo /;
+    pk qw/ id bar /;
+    columns qw/ id name foo bar /;
     inflate 'name' => sub {
         my ($col_value) = @_;
         return Mock::Inflate::Name->new(name => $col_value);
@@ -24,6 +24,14 @@ table {
         my ($col_value) = @_;
         return ref $col_value ? $col_value->name : $col_value . '_deflate';
     };
+    inflate 'bar' => sub {
+        my ($col_value) = @_;
+        return Mock::Inflate::Name->new(name => $col_value);
+    };
+    deflate 'bar' => sub {
+        my ($col_value) = @_;
+        return ref $col_value ? $col_value->name : $col_value . '_deflate';
+    };
 };
 
 1;
